"use strict";
/// <reference types="node" />
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
class MemorySlidingWindowRateLimiter extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        this.options = options;
        this.buckets = {};
        this.timers = {};
        this.interval = Number(options.interval) || 60;
    }
    check(key, limit) {
        this.bucketExpireNow(key);
        const usage = this.buckets[key].length;
        if (usage > limit) {
            return Promise.resolve(-limit);
        }
        else {
            return Promise.resolve(usage);
        }
    }
    reserve(key, limit) {
        const now = this.bucketExpireNow(key);
        const usage = this.buckets[key].length;
        if (usage >= limit) {
            return Promise.resolve(-limit);
        }
        else {
            this.buckets[key].push(now);
            if (this.timers[key]) {
                clearTimeout(this.timers[key]);
            }
            this.timers[key] = setTimeout(() => {
                delete this.buckets[key];
                delete this.timers[key];
            }, this.interval * 1000);
            return Promise.resolve(now);
        }
    }
    cancel(key, ts) {
        this.bucketExpireNow(key);
        let canceled = 0;
        const position = this.buckets[key].indexOf(ts);
        if (position !== -1) {
            canceled = this.buckets[key].splice(position, 1).length;
        }
        return Promise.resolve(canceled);
    }
    destroy() {
        for (const key of Object.keys(this.timers)) {
            clearTimeout(this.timers[key]);
        }
    }
    bucketExpireNow(key) {
        const now = new Date().getTime();
        this.buckets[key] = this.buckets[key] ? this.buckets[key].filter((ts) => now - ts < this.interval * 1000) : [];
        return now;
    }
}
exports.MemorySlidingWindowRateLimiter = MemorySlidingWindowRateLimiter;
exports.default = MemorySlidingWindowRateLimiter;
