/// <reference types="node" />
import { EventEmitter } from 'events';
import IORedis = require('ioredis');
import { SlidingWindowRateLimiterBackend, SlidingWindowRateLimiterBackendOptions } from './sliding-window-rate-limiter-backend';
declare type ms = number;
declare type s = number;
declare const enum LimiterMode {
    Check = 0,
    Reserve = 1,
    Cancel = 2
}
export interface Redis extends IORedis.Redis {
    limiter(key: string, mode: LimiterMode, interval: s, limit: number, ts: ms): Promise<number | ms>;
}
export interface RedisSlidingWindowRateLimiterOptions extends SlidingWindowRateLimiterBackendOptions {
    redis?: Redis | string;
    interval?: s;
    operationTimeout?: ms;
}
export declare class RedisSlidingWindowRateLimiter extends EventEmitter implements SlidingWindowRateLimiterBackend {
    readonly options: RedisSlidingWindowRateLimiterOptions;
    readonly redis: Redis;
    readonly interval: s;
    readonly operationTimeout: ms;
    constructor(options?: RedisSlidingWindowRateLimiterOptions);
    check(key: string, limit: number): Promise<number>;
    reserve(key: string, limit: number): Promise<number | ms>;
    cancel(key: string, ts: ms): Promise<number | ms>;
    destroy(): void;
    protected limiter(key: string, mode: LimiterMode, interval: s, limit: number, ts: ms): Promise<number | ms>;
    protected promiseWithTimeout<T>(operationPromise: Promise<T>): Promise<T>;
}
export default RedisSlidingWindowRateLimiter;
