/// <reference types="node" />
import { RedisSlidingWindowRateLimiter, RedisSlidingWindowRateLimiterOptions } from './redis-sliding-window-rate-limiter';
import { SlidingWindowRateLimiterBackend } from './sliding-window-rate-limiter';
declare type ms = number;
export interface SafeRedisSlidingWindowRateLimiterOptions extends RedisSlidingWindowRateLimiterOptions {
    safe?: true;
    reuseRedisAfter?: number;
    defaultResponse?: number;
}
export interface SafeRedisSlidingWindowRateLimiter {
    addListener(event: 'error', listener: (err: Error) => void): this;
    emit(event: 'error', error: Error): boolean;
    listeners(event: 'error'): Array<(err: Error) => void>;
    off(event: 'error', listener: (err: Error) => void): this;
    on(event: 'error', listener: (err: Error) => void): this;
    once(event: 'error', listener: (err: Error) => void): this;
    prependListener(event: 'error', listener: (err: Error) => void): this;
    prependOnceListener(event: 'error', listener: (err: Error) => void): this;
    removeListener(event: 'error', listener: (err: Error) => void): this;
}
export declare class SafeRedisSlidingWindowRateLimiter extends RedisSlidingWindowRateLimiter implements SlidingWindowRateLimiterBackend {
    readonly options: SafeRedisSlidingWindowRateLimiterOptions;
    readonly reuseRedisAfter: number;
    readonly defaultResponse: number;
    protected redisServiceAvailable: boolean;
    protected reconnectTimer?: NodeJS.Timer;
    constructor(options?: SafeRedisSlidingWindowRateLimiterOptions);
    check(key: string, limit: number): Promise<number>;
    reserve(key: string, limit: number): Promise<number | ms>;
    cancel(key: string, ts: ms): Promise<number | ms>;
    destroy(): void;
    protected promiseErrorHandler(originPromise: Promise<number | ms>, defaultResponse: number): Promise<number | ms>;
    protected handleError(error: Error): void;
    protected markServiceAsUnavailable(): void;
}
export default SafeRedisSlidingWindowRateLimiter;
