"use strict";
/// <reference types="node" />
Object.defineProperty(exports, "__esModule", { value: true });
const redis_sliding_window_rate_limiter_1 = require("./redis-sliding-window-rate-limiter");
class SafeRedisSlidingWindowRateLimiter extends redis_sliding_window_rate_limiter_1.RedisSlidingWindowRateLimiter {
    constructor(options = {}) {
        super(options);
        this.options = options;
        this.redisServiceAvailable = true;
        this.reconnectTimer = undefined;
        this.reuseRedisAfter = Number(options.reuseRedisAfter) || 2000;
        this.defaultResponse = Number(options.defaultResponse) || 0;
    }
    check(key, limit) {
        if (this.redisServiceAvailable) {
            return this.promiseErrorHandler(super.check(key, limit), this.defaultResponse);
        }
        else {
            return Promise.resolve(this.defaultResponse);
        }
    }
    reserve(key, limit) {
        if (this.redisServiceAvailable) {
            return this.promiseErrorHandler(super.reserve(key, limit), this.defaultResponse);
        }
        else {
            return Promise.resolve(this.defaultResponse);
        }
    }
    cancel(key, ts) {
        if (this.redisServiceAvailable) {
            return this.promiseErrorHandler(super.cancel(key, ts), this.defaultResponse);
        }
        else {
            return Promise.resolve(this.defaultResponse);
        }
    }
    destroy() {
        super.destroy();
        if (this.reconnectTimer) {
            clearTimeout(this.reconnectTimer);
        }
        this.removeAllListeners('error');
    }
    promiseErrorHandler(originPromise, defaultResponse) {
        return new Promise((resolve) => {
            originPromise.then((successValue) => {
                resolve(successValue);
            }, (error) => {
                this.handleError(error);
                resolve(defaultResponse);
            });
        });
    }
    handleError(error) {
        this.markServiceAsUnavailable();
        this.emit('error', error);
    }
    markServiceAsUnavailable() {
        this.redisServiceAvailable = false;
        if (this.reconnectTimer) {
            clearTimeout(this.reconnectTimer);
        }
        this.reconnectTimer = setTimeout(() => {
            this.redisServiceAvailable = true;
        }, this.reuseRedisAfter);
    }
}
exports.SafeRedisSlidingWindowRateLimiter = SafeRedisSlidingWindowRateLimiter;
exports.default = SafeRedisSlidingWindowRateLimiter;
